from flask import request, jsonify, render_template
from . import app, db
from .models import Assessment, Question, StudentResponse
from datetime import datetime
from sqlalchemy import func

# 定义路由
@app.route('/')
def index():
    return render_template('index.html')

@app.route('/create_assessment', methods=['GET', 'POST'])
def create_assessment():
    if request.method == 'POST':
        data = request.json
        new_assessment = Assessment(
            title=data['title'],
            type=data['type'],
            deadline=datetime.strptime(data['deadline'], '%Y-%m-%d %H:%M:%S') if data.get('deadline') else None
        )
        db.session.add(new_assessment)
        db.session.commit()
        return jsonify({"message": "Assessment created", "id": new_assessment.id}), 201
    else:
        return render_template('create_assessment.html')

@app.route('/add_question', methods=['POST'])
def add_question():
    data = request.json
    new_question = Question(
        text=data['text'],
        type=data['type'],
        correct_answer=data['correct_answer'],
        assessment_id=data['assessment_id']
    )
    db.session.add(new_question)
    db.session.commit()
    return jsonify({"message": "Question added", "id": new_question.id}), 201

# 显示评估选择页面
@app.route('/take_assessment', methods=['GET'])
def take_assessment():
    return render_template('take_assessment.html')

#显示形成性评估页面
@app.route('/take_assessment/formative_test', methods=['GET'])
def formative_test():
    # 查询所有形成性评估
    assessment = Assessment.query.filter_by(type='formative').first()
    if assessment:
        questions = Question.query.filter_by(assessment_id=assessment.id).all()
        return render_template('formative_test.html', assessment=assessment, questions=questions)
    else:
        return "No formative assessment available.", 404

# 显示总结性评估页面
@app.route('/take_assessment/summative_test', methods=['GET'])
def summative_test():
    # 查询所有总结性评估
    assessment = Assessment.query.filter_by(type='summative').first()
    if assessment:
        questions = Question.query.filter_by(assessment_id=assessment.id).all()
        return render_template('summative_test.html', assessment=assessment, questions=questions)
    else:
        return "No summative assessment available.", 404

# @app.route('/take_assessment/<int:assessment_id>', methods=['GET'])
# def take_assessment(assessment_id):
#     assessment = Assessment.query.get_or_404(assessment_id)
#     questions = Question.query.filter_by(assessment_id=assessment_id).all()
#     return render_template('take_assessment.html', assessment=assessment, questions=questions)

# 管理问题页面
@app.route('/manage_questions', methods=['GET'])
def manage_questions():
    return render_template('manage_questions.html')

@app.route('/submit_response', methods=['POST'])
def submit_response():
    data = request.json
    question = Question.query.get_or_404(data['question_id'])
    is_correct = data['response'] == question.correct_answer
    new_response = StudentResponse(
        student_id=data['student_id'],
        question_id=data['question_id'],
        response=data['response'],
        is_correct=is_correct
    )
    db.session.add(new_response)
    db.session.commit()
    return jsonify({"message": "Response submitted", "is_correct": is_correct}), 201

@app.route('/get_results/<int:assessment_id>', methods=['GET'])
def get_results(assessment_id):
    assessment = Assessment.query.get_or_404(assessment_id)
    responses = StudentResponse.query.join(Question).filter(Question.assessment_id == assessment_id).all()
    results = [{"student_id": r.student_id, "question_id": r.question_id, "response": r.response, "is_correct": r.is_correct} for r in responses]
    
    # 计算统计信息
    total_students = db.session.query(StudentResponse.student_id).distinct().count()
    average_score = db.session.query(func.avg(StudentResponse.is_correct)).filter(Question.assessment_id == assessment_id).scalar()
    most_incorrect_question = db.session.query(Question.text, func.count(StudentResponse.id)).join(StudentResponse).filter(StudentResponse.is_correct == False).group_by(Question.text).order_by(func.count(StudentResponse.id).desc()).first()
    
    statistics = {
        "total_students": total_students,
        "average_score": average_score,
        "most_incorrect_question": most_incorrect_question[0] if most_incorrect_question else None
    }
    
    return jsonify({"results": results, "statistics": statistics}), 200